package resourcePredictor;

import mocks.ApplicationHistoryDatabaseMock;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import dataTypes.ResourceType;

public class IntervalPredictorTest {

	private ApplicationHistoryDatabaseMock mock = null;

	private IntervalPredictor predictor = null;

	private double epsilon = 10e-5;

	@Before
	public void setUp() throws Exception {
		mock = new ApplicationHistoryDatabaseMock();
		predictor = new IntervalPredictor(mock);
	}

	@Test
	public void predictionShouldBeGreaterThanMean() {
		mock.setSize(1);
		mock.setMean(ResourceType.executionTime, 1);
		mock.setMean(ResourceType.memory, 1);
		mock.setStandardDeviation(ResourceType.executionTime, 1);
		mock.setStandardDeviation(ResourceType.memory, 1);

		Prediction prediction = predictor.predictNext("testApp");
		Assert.assertTrue(prediction.getExecutionTime() > 1);
		Assert.assertTrue(prediction.getMemory() > 1);
	}

	@Test
	public void predictionShouldBeEqualMean() {
		mock.setSize(1);
		mock.setMean(ResourceType.executionTime, 1.5);
		mock.setMean(ResourceType.memory, 1);
		mock.setStandardDeviation(ResourceType.executionTime, 0);
		mock.setStandardDeviation(ResourceType.memory, 0);

		Prediction prediction = predictor.predictNext("testApp");
		Assert.assertEquals(1.5, prediction.getExecutionTime(), epsilon);
		Assert.assertEquals(1, prediction.getMemory());
	}
	
	@Test
	public void predictionOfFractionaryValueShouldBeTheCeil() {
		mock.setSize(1);
		mock.setMean(ResourceType.executionTime, 1);
		mock.setMean(ResourceType.memory, 1.5);
		mock.setStandardDeviation(ResourceType.executionTime, 0);
		mock.setStandardDeviation(ResourceType.memory, 0);
		
		Prediction prediction = predictor.predictNext("testApp");
		Assert.assertEquals(1, prediction.getExecutionTime(), epsilon);
		Assert.assertEquals(2, prediction.getMemory());
	}

}
