#include "ResourceMonitor.hpp"

#include "NodeStaticInformation.hpp"

#include <cstdlib>

ResourceMonitor::ResourceMonitor() {
	NodeStaticInformation nodeStaticInformation;
	cores = nodeStaticInformation.getNumberOfCores();
	totalMemory = nodeStaticInformation.getTotalRAM();
	pthread_mutex_init(&resourceMonitorLock, NULL);
}

ResourceMonitor::~ResourceMonitor() {
	// TODO Auto-generated destructor stub
}

void ResourceMonitor::startCollectingData() {
	pthread_create(&collectingThread, NULL, ResourceMonitor::collectDataThread,
			(void*) this);
}

void ResourceMonitor::addPid(int pid) {
	ProcessResources processResource = ProcessResources();
	processResource.cpu = 0;
	processResource.memory = 0;
	pthread_mutex_lock(&resourceMonitorLock);
	processInformations[pid] = processResource;
	pthread_mutex_unlock(&resourceMonitorLock);
	std::cerr << "Monitoring pid " << pid << std::endl;
}

void ResourceMonitor::removePid(int pid) {
	pthread_mutex_lock(&resourceMonitorLock);
	processInformations.erase(pid);
	pthread_mutex_unlock(&resourceMonitorLock);
}

void ResourceMonitor::findColumns(string header) {
	cpuIndex = header.find("%CPU");
	memIndex = header.find("%MEM");
	pidIndex = header.find("PID");
}

string ResourceMonitor::getColumnValue(string line, int index) {
	int begin = line.find_last_of(" \t", index);
	int end = line.find_first_of(" \t", index);

	return line.substr(begin, end);
}

void ResourceMonitor::collectData() {
	FILE *stream;
	char *l = (char *) malloc(MAX_BUFFER * sizeof(char));
	string header;
	string line;
	string buffer;
	double memTmp = 0.0;
	double cpu = 0.0;
	long memory = 0;

	// If there is no process to be monitored
	if (processInformations.empty()) {
		return;
	}

	stream = popen("ps aux", "r");
	l = fgets(l, MAX_BUFFER, stream);
	header.append(l);
	this->findColumns(header);

	for (l = fgets(l, MAX_BUFFER, stream); l != NULL; l = fgets(l, MAX_BUFFER,
			stream)) {
		line.assign(l);

		int pid = atoi(this->getColumnValue(line, pidIndex).c_str());
		if (processInformations.find(pid) == processInformations.end()) {
			continue;
		}
		cpu = atof(this->getColumnValue(line, cpuIndex).c_str());
		memTmp = atof(this->getColumnValue(line, memIndex).c_str());
		cpu = cpu / (cores * 100.0);
		memTmp = (long) (memTmp * totalMemory / 100.0);
		if(memTmp > memory) {
			memory = memTmp;
		}

		pthread_mutex_lock(&resourceMonitorLock);
		processInformations[pid].cpu = cpu;
		processInformations[pid].memory = memory;
		pthread_mutex_unlock(&resourceMonitorLock);
	}
	pclose(stream);

}

ProcessResources *ResourceMonitor::getProcessResources(int pid) {
	ProcessResources *pr;
	pthread_mutex_lock(&resourceMonitorLock);
	if (processInformations.find(pid) == processInformations.end()) {
		pr = NULL;
	} else {
		pr = &processInformations[pid];
	}
	pthread_mutex_unlock(&resourceMonitorLock);
	return pr;

}

void *ResourceMonitor::collectDataThread(void *resourceMonitor) {
	while (true) {
		sleep(SLEEP_INTERVAL);
		((ResourceMonitor*) resourceMonitor)->collectData();
	}
}

